import strawberry
from typing import Optional

from kramlin.oauth.graphql.types import OAuthConfiguration, OAuthPublicConfiguration


@strawberry.type
class SSOConfiguration:
    oAuthEnabled: Optional[bool] = None
    oAuth: Optional[OAuthConfiguration] = None


@strawberry.type
class PublicSSOConfiguration:
    oAuth: Optional[OAuthPublicConfiguration] = None


__all__ = [
    'SSOConfiguration',
    'PublicSSOConfiguration'
]



from .provider import OAuthProviderConfiguration


class OAuthClientConfiguration:

    def __init__(
        self,
        clientID: str,
        clientSecret: str,
        redirectURI: str,
        provider: OAuthProviderConfiguration,
        scopes: str = None
    ):
        self.clientID = clientID
        self.clientSecret = clientSecret
        self.redirectURI = redirectURI
        self.scopes = scopes
        self.provider = provider


__all__ = [
    'OAuthClientConfiguration'
]



from typing import Optional

import strawberry


@strawberry.input
class OAuthConfigurationInput:
    clientID: str
    clientSecret: str
    authorizationEndpoint: str
    tokenEndpoint: str
    userInfoEndpoint: str
    revocationEndpoint: Optional[str] = None
    scopes: Optional[str] = None


__all__ = [
    'OAuthConfigurationInput'
]



import strawberry
from enum import Enum


@strawberry.enum
class EmailServerTypeEnum(Enum):
    SMTP = "SMTP"


@strawberry.enum
class EmailConnectionSecurityEnum(Enum):
    NONE = "NONE"
    SSL = "SSL"
    TLS = "TLS"


__all__ = [
    'EmailServerTypeEnum',
    'EmailConnectionSecurityEnum'
]



import strawberry
from typing import Optional

from .enum import EmailConnectionSecurityEnum, EmailServerTypeEnum


@strawberry.input
class EmailSMTPSettingsInput:
    username: Optional[str] = None
    password: Optional[str] = None
    email: Optional[str] = None
    address: Optional[str] = None
    port: Optional[int] = None
    connectionSecurity: Optional[EmailConnectionSecurityEnum] = None


@strawberry.input
class MailingConfigurationInput:
    enforceEmailVerification: Optional[bool] = None
    serverType: Optional[EmailServerTypeEnum] = None
    smtpSettings: Optional[EmailSMTPSettingsInput] = None


__all__ = [
    'EmailSMTPSettingsInput',
    'MailingConfigurationInput'
]



import strawberry
from typing import Optional

from .enum import EmailConnectionSecurityEnum


@strawberry.type
class EmailSMTPSettings:
    username: Optional[str] = None
    password: Optional[str] = None
    email: Optional[str] = None
    address: Optional[str] = None
    port: Optional[int] = None
    connectionSecurity: Optional[EmailConnectionSecurityEnum] = None


@strawberry.type
class MailingConfiguration:
    enforceEmailVerification: Optional[bool] = None
    serverType: Optional[str] = None
    smtpSettings: Optional[EmailSMTPSettings] = None


__all__ = [
    'EmailSMTPSettings',
    'MailingConfiguration'
]



import strawberry
from typing import Optional

from kramlin.oauth.graphql.inputs import OAuthConfigurationInput


@strawberry.input
class SSOConfigurationInput:
    oAuthEnabled: Optional[bool] = None
    oAuth: Optional[OAuthConfigurationInput] = None


__all__ = [
    'SSOConfigurationInput'
]



from django.conf import settings
from django.core.cache import caches

MANAGER_CACHE_NAME = (
    settings.PLATFORM_MANAGE_CACHE_NAME if hasattr(settings, 'PLATFORM_MANAGE_CACHE_NAME') else 'platform_settings'
)


class BasePlatformManager:

    def __init__(self):
        pass

    @staticmethod
    def get_config():
        cache = caches["default"].get(MANAGER_CACHE_NAME)
        if cache:
            return cache

    @staticmethod
    def set_config(obj):
        caches["default"].set(MANAGER_CACHE_NAME, obj, timeout=None)

    @staticmethod
    def get_val(obj, key, default=None):
        if (
            obj and (
                (type(obj) == dict and key in obj)
                or (type(obj) == object and hasattr(obj, key))
            ) and obj[key] is not None
        ):
            return obj[key]
        return default

    @staticmethod
    def get_if_val(val):
        if val and len(val) > 0:
            return val
        return None


__all__ = [
    'BasePlatformManager',
]



from graphql import GraphQLError


class APIError(GraphQLError):

    def __init__(self, message, code=None, *args, **kwargs):
        super().__init__(message, *args, **kwargs)
        self.locations = []
        self.path = []
        self.code = code
        self.message = message

    @property
    def formatted(self):
        return {
            "message": self.message or "An unknown error occurred.",
            "code": self.code or "UNKNOWN_ERROR",
        }


__all__ = [
    'APIError',
]



from typing import Optional

from requests import post, get
from kramlin.exceptions import APIError

from .client import OAuthClientConfiguration
from .provider import OAuthProviderConfiguration


class OAuthTokenResponse:
    def __init__(self, access_token: str, token_type: str = 'Bearer', refresh_token: str = None):
        self.access_token = access_token
        self.refresh_token = refresh_token
        self.token_type = token_type

    def __repr__(self):
        return f"<OAuthTokenResponse {self.access_token}>"


class OAuthLoginResponse:
    def __init__(self, token: OAuthTokenResponse, user_info: dict):
        self.token = token
        self.user_info = user_info

    def __repr__(self):
        return f"<OAuthLoginResponse {self.token} {self.user_info}>"


class OAuthHandler:

    def __init__(self, client: OAuthClientConfiguration):
        self.clientID = client.clientID
        self.clientSecret = client.clientSecret
        self.redirect_uri = client.redirectURI
        self.authVar = client.provider.authVar
        self.provider: OAuthProviderConfiguration = client.provider

    def get_token_from_auth_code(self, code: str) -> Optional[OAuthTokenResponse]:
        resp = post(
            url=self.provider.tokenEndpoint,
            data={
                "grant_type": "authorization_code",
                "code": code,
                "redirect_uri": self.redirect_uri,
                "client_id": self.clientID,
                "client_secret": self.clientSecret,
            }
        )
        if resp.status_code != 200:
            print("Failed to verify authorization code.")
            print(resp.text)
            raise APIError(
                code="OAUTH_FAILED", message="Failed to verify authorizationCode."
            )
        resp_type = resp.headers.get("Content-Type")
        refresh_token = None
        if "application/json" in resp_type:
            data = resp.json()
            access_token = data["access_token"]
            if 'refreshToken' in data and len(data['refreshToken']) > 0:
                refresh_token = data['refresh_token']
            if "token_type" in data and data["token_type"]:
                self.authVar = data["token_type"]
        elif "application/x-www-form-urlencoded" in resp_type:
            from urllib.parse import parse_qs
            data = parse_qs(resp.text)
            if "token_type" in data and data["token_type"]:
                self.authVar = data["token_type"][0]
            access_token = data["access_token"][0]
            if 'refreshToken' in data and len(data['refreshToken']) > 0:
                refresh_token = data["refresh_token"][0]
        else:
            raise APIError(
                code="OAUTH_FAILED",
                message="Failed to verify authorization code. Unknown response type.",
            )
        return OAuthTokenResponse(
            access_token=access_token,
            refresh_token=refresh_token,
            token_type=self.authVar
        )

    def get_user_info_with_token(self, access_token: str) -> dict:
        resp = get(
            url=self.provider.userInfoEndpoint,
            headers={
                "Authorization": f"{self.authVar} {access_token}",
            },
        )
        if resp.status_code != 200:
            print("Failed to retrieve user info.")
            print(resp.text)
            raise APIError(code="OAUTH_FAILED", message="Failed to retrieve User Info.")
        resp_type = resp.headers.get("Content-Type")
        if "application/json" in resp_type:
            return resp.json()
        raise APIError(
            code="OAUTH_FAILED",
            message="Failed to retrieve User Info. Unknown response type.",
        )

    def perform_login(self, code: str) -> OAuthLoginResponse:
        # Give authorization code got from frontend with ClientSecret to generate access token
        token_data = self.get_token_from_auth_code(code=code)
        # Use the generated access token to fetch the details about the user, especially their email
        user_info = self.get_user_info_with_token(access_token=token_data.access_token)
        # email is required to create or login to a user account
        if "email" not in user_info or len(user_info["email"]) < 1:
            raise APIError(
                code="OAUTH_FAILED",
                message="Unable to obtain email address from OAuth provider.",
            )
        return OAuthLoginResponse(
            token=token_data,
            user_info=user_info,
        )


__all__ = [
    "OAuthHandler",
    "OAuthLoginResponse"
]



#  Copyright © 2022 Traboda CyberLabs Private Limited.
#  All Rights Reserved.
from typing import Optional, Union, List

from .graphql.types import MailingConfiguration
from .smtp import SMTPManager
from kramlin.mailing.graphql.enum import EmailServerTypeEnum
from kramlin.mailing.graphql.inputs import EmailSMTPSettingsInput


class MailingManager:
    def __init__(
        self,
        enforceEmailVerification: Optional[bool] = False,
        serverType: Optional[EmailServerTypeEnum] = EmailServerTypeEnum.SMTP,
        smtpSettings: Optional[Union[EmailSMTPSettingsInput, dict]] = None
    ):
        self.enforceEmailVerification = enforceEmailVerification
        self.serverType: EmailServerTypeEnum = serverType
        self.smtpSettings: Optional[SMTPManager] = None
        if smtpSettings:
            if smtpSettings and type(smtpSettings) == dict:
                self.smtpSettings = EmailSMTPSettingsInput(**smtpSettings)
            self.smtpSettings = SMTPManager(
                username=smtpSettings.username,
                password=smtpSettings.password,
                email=smtpSettings.email,
                address=smtpSettings.address,
                port=smtpSettings.port,
                connectionSecurity=smtpSettings.connectionSecurity
            )

    def get_from_email(self):
        if self.serverType == EmailServerTypeEnum.SMTP and self.smtpSettings:
            return self.smtpSettings.email

    def get_email_backend(self, fail_silently=False):
        if self.serverType == EmailServerTypeEnum.SMTP and self.smtpSettings:
            return self.smtpSettings.get_backend(fail_silently=fail_silently)

    def send_email(
        self,
        subject: str,
        message: str,
        recipient_list: List[str],
        html_message: Optional[str] = None,
        fail_silently: Optional[bool] = False,
    ) -> int:
        backend = self.get_email_backend(fail_silently=fail_silently)
        from_email = self.get_from_email()

        from django.core.mail import EmailMultiAlternatives

        mail = EmailMultiAlternatives(
            connection=backend,
            from_email=from_email,
            to=recipient_list,
            subject=subject,
            body=message,
        )
        if html_message:
            mail.attach_alternative(html_message, "text/html")
        return mail.send()

    def graphql(self) -> MailingConfiguration:
        return MailingConfiguration(
            enforceEmailVerification=self.enforceEmailVerification,
            serverType=self.serverType,
            smtpSettings=self.smtpSettings.graphql() if self.smtpSettings else None,
        )

    def dict(self):
        return {
            "enforceEmailVerification": self.enforceEmailVerification,
            "serverType": self.serverType.value if self.serverType else EmailServerTypeEnum.SMTP.value,
            "smtpServer": self.smtpSettings.dict(),
        }


__all__ = [
    "MailingManager",
]



from typing import Optional, Union
from django.conf import settings

from kramlin.oauth.graphql.inputs import OAuthConfigurationInput
from kramlin.oauth.graphql.types import OAuthConfiguration, OAuthPublicConfiguration
from kramlin.oauth.utils import OAuthClientConfiguration, OAuthHandler, OAuthProviderConfiguration
from .graphql.types import SSOConfiguration, PublicSSOConfiguration


class OAuthAuthenticateResponse:
    def __init__(self, user_info: dict, user=None):
        self.user_info = user_info
        self.user = user


class SSOManager:

    def __init__(
        self,
        oAuthEnabled: Optional[bool] = False,
        oAuth: Optional[Union[OAuthConfigurationInput, dict]] = None,
    ):
        self.oAuthEnabled: bool = oAuthEnabled
        self.oAuth: Optional[OAuthClientConfiguration] = None
        if oAuth:
            self.update_oauth(oAuth)

    def update_oauth(self, oAuth: OAuthConfigurationInput):
        if type(oAuth) == dict:
            oAuth = OAuthConfigurationInput(**oAuth)
        self.oAuth = OAuthClientConfiguration(
            clientID=oAuth.clientID,
            clientSecret=oAuth.clientSecret,
            scopes=oAuth.scopes,
            redirectURI=(
                settings.SSO_OAUTH_REDIRECT_URL if hasattr(settings, 'SSO_OAUTH_REDIRECT_URL')
                else "http://localhost/oauth-callback"
            ),
            provider=OAuthProviderConfiguration(
                name='SSO',
                authorizationEndpoint=oAuth.authorizationEndpoint,
                tokenEndpoint=oAuth.tokenEndpoint,
                revocationEndpoint=oAuth.revocationEndpoint,
                userInfoEndpoint=oAuth.userInfoEndpoint,
            )
        )

    def authenticate_user(self, code) -> Optional[OAuthAuthenticateResponse]:
        if not (self.oAuthEnabled and self.oAuth):
            return None
        handler = OAuthHandler(client=self.oAuth)
        data = handler.perform_login(code=code)

        from django.contrib.auth import get_user_model
        User = get_user_model()

        try:
            # If the user with the OAuth'ed email already has an account
            user = User.objects.get(email__iexact=data.user_info['email'])
            if hasattr(User, 'ssoData'):
                user.ssoData = data.user_info
                user.save()
            return OAuthAuthenticateResponse(
                user_info=data.user_info,
                user=user,
            )
        except User.DoesNotExist:
            return OAuthAuthenticateResponse(
                user_info=data.user_info,
            )

    def graphql(self) -> SSOConfiguration:
        return SSOConfiguration(
            oAuthEnabled=self.oAuthEnabled,
            oAuth=OAuthConfiguration(
                redirectURL=self.oAuth.redirectURI,
                clientID=self.oAuth.clientID,
                clientSecret=self.oAuth.clientSecret,
                scopes=self.oAuth.scopes,
                authorizationEndpoint=self.oAuth.provider.authorizationEndpoint,
                tokenEndpoint=self.oAuth.provider.tokenEndpoint,
                userInfoEndpoint=self.oAuth.provider.userInfoEndpoint,
                revocationEndpoint=self.oAuth.provider.revocationEndpoint
            ) if self.oAuth else None,
        )

    def public_graphql(self) -> PublicSSOConfiguration:
        return PublicSSOConfiguration(
            oAuth=OAuthPublicConfiguration(
                clientID=self.oAuth.clientID,
                authorizationURL=self.oAuth.provider.authorizationEndpoint,
                scopes=self.oAuth.scopes,
                redirectURL=self.oAuth.redirectURI,
            ) if self.oAuthEnabled and self.oAuth else None,
        )

    def dict(self) -> dict:
        return {
            'oAuthEnabled': self.oAuthEnabled,
            'oAuth': {
                'clientID': self.oAuth.clientID,
                'clientSecret': self.oAuth.clientSecret,
                'scopes': self.oAuth.scopes,
                'authorizationEndpoint': self.oAuth.provider.authorizationEndpoint,
                'tokenEndpoint': self.oAuth.provider.tokenEndpoint,
                'userInfoEndpoint': self.oAuth.provider.userInfoEndpoint,
                'revocationEndpoint': self.oAuth.provider.revocationEndpoint
            } if self.oAuth else None,
        }


__all__ = [
    'SSOManager',
]



from typing import Union, Optional

from django.core.mail.backends.smtp import EmailBackend

from kramlin.mailing.graphql.enum import EmailConnectionSecurityEnum
from kramlin.mailing.graphql.types import EmailSMTPSettings


class SMTPManager:

    def __init__(
        self,
        username: str = None,
        password: str = None,
        email: str = None,
        address: str = None,
        port: int = 587,
        connectionSecurity: Optional[Union[EmailConnectionSecurityEnum, str]] = EmailConnectionSecurityEnum.NONE,
    ):
        self.username: str = username
        self.password: str = password
        self.email: str = email
        self.address: str = address
        self.port: int = port
        if connectionSecurity and type(connectionSecurity) == str:
            connectionSecurity = (
                EmailConnectionSecurityEnum.SSL if connectionSecurity == 'SSL' else
                EmailConnectionSecurityEnum.TLS if connectionSecurity == 'TLS' else
                EmailConnectionSecurityEnum.NONE
            )
        self.connectionSecurity: EmailConnectionSecurityEnum = connectionSecurity

    def get_backend(self, fail_silently=False) -> EmailBackend:
        return EmailBackend(
            host=self.address,
            port=self.port,
            username=self.username,
            password=self.password,
            use_ssl=self.connectionSecurity == EmailConnectionSecurityEnum.SSL,
            use_tls=self.connectionSecurity == EmailConnectionSecurityEnum.TLS,
            fail_silently=fail_silently,
        )

    def graphql(self) -> EmailSMTPSettings:
        return EmailSMTPSettings(
            username=self.username,
            password=self.password,
            email=self.email,
            address=self.address,
            port=self.port,
            connectionSecurity=self.connectionSecurity,
        )

    def dict(self) -> dict:
        return {
            "username": self.username,
            "password": self.password,
            "email": self.email,
            "address": self.address,
            "port": self.port,
            "connectionSecurity": (
                self.connectionSecurity.value if self.connectionSecurity else EmailConnectionSecurityEnum.NONE.value
            )
        }


__all__ = [
    'SMTPManager',
]



from typing import Optional


class OAuthProviderConfiguration:

    def __init__(
        self,
        name: str,
        authorizationEndpoint: str,
        tokenEndpoint: str,
        userInfoEndpoint: str,
        revocationEndpoint: str = None,
        authVar: Optional[str] = "Bearer"
    ):
        self.name = name
        self.authorizationEndpoint = authorizationEndpoint
        self.tokenEndpoint = tokenEndpoint
        self.userInfoEndpoint = userInfoEndpoint
        self.revocationEndpoint = revocationEndpoint
        self.authVar = authVar


GOOGLE_OAUTH_CONFIG = OAuthProviderConfiguration(
    name='google',
    authorizationEndpoint='https://accounts.google.com/o/oauth2/v2/auth',
    tokenEndpoint='https://oauth2.googleapis.com/token',
    userInfoEndpoint='https://www.googleapis.com/oauth2/v1/userinfo?alt=json',
    authVar='Bearer'
)

GITHUB_OAUTH_CONFIG = OAuthProviderConfiguration(
    name='github',
    authorizationEndpoint='https://github.com/login/oauth/authorize',
    tokenEndpoint='https://github.com/login/oauth/access_token',
    userInfoEndpoint='https://api.github.com/user',
    authVar='token'
)

TWITTER_OAUTH_CONFIG = OAuthProviderConfiguration(
    name='twitter',
    authorizationEndpoint='https://twitter.com/i/oauth2/authorize',
    tokenEndpoint='https://api.twitter.com/oauth2/token',
    userInfoEndpoint='https://api.twitter.com/2/users/me',
    authVar='Bearer'
)

TRABODA_OAUTH_CONFIG = OAuthProviderConfiguration(
    name='traboda',
    authorizationEndpoint='https://traboda.com/oauth/authorize',
    tokenEndpoint='https://app.traboda.com/api/auth/token/',
    userInfoEndpoint='https://app.traboda.com/api/auth/user/',
    authVar='Bearer'
)


def get_provider_configuration(provider: str) -> OAuthProviderConfiguration:
    if provider.lower() == 'google':
        return GOOGLE_OAUTH_CONFIG
    elif provider.lower() == 'github':
        return GITHUB_OAUTH_CONFIG
    elif provider.lower() == 'twitter':
        return TWITTER_OAUTH_CONFIG
    elif provider.lower() == 'traboda':
        return TRABODA_OAUTH_CONFIG
    else:
        raise ValueError(f'Unknown OAuth provider: {provider}')


__all__ = [
    'OAuthProviderConfiguration',
    'GITHUB_OAUTH_CONFIG',
    'GOOGLE_OAUTH_CONFIG',
    'TWITTER_OAUTH_CONFIG',
    'TRABODA_OAUTH_CONFIG',
    'get_provider_configuration',
]




import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md'), encoding='utf8').read()

# Allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup()


import strawberry

from typing import Optional


@strawberry.type
class OAuthPublicConfiguration:
    clientID: Optional[str] = None
    authorizationURL: Optional[str] = None
    scopes: Optional[str] = None
    redirectURL: Optional[str] = None


@strawberry.type
class OAuthProvider:
    name: Optional[str] = None
    authorizationEndpoint: Optional[str] = None
    tokenEndpoint: Optional[str] = None
    revocationEndpoint: Optional[str] = None
    userInfoEndpoint: Optional[str] = None
    redirectURL: Optional[str] = None
    scopes: Optional[str] = None


@strawberry.type
class OAuthConfiguration(OAuthProvider):
    clientID: Optional[str] = None
    clientSecret: Optional[str] = None


__all__ = [
    "OAuthProvider",
    "OAuthConfiguration",
    "OAuthPublicConfiguration",
]
